from socket import socket, AF_INET, SOCK_STREAM, MSG_WAITALL
from contextlib import closing, nested
from functools import partial
from struct import unpack, calcsize
import sys


HEADER_SIZE = calcsize('!l')
log = partial(print, file=sys.stderr)


hostname, port = sys.argv[1], int(sys.argv[2])

s = socket(AF_INET, SOCK_STREAM)
s.bind((hostname, port))

s.listen(1)
log('listening on port', port)

(conn, addr) = s.accept()
log('accepted client from', addr)


with nested(closing(s), closing(conn)):
    while True:
        msg_header = conn.recv(HEADER_SIZE, MSG_WAITALL)
        if len(msg_header) < HEADER_SIZE:
            log('unable to recieve full header')
            break

        (nbytes_left,)  = unpack('!l', msg_header)
        if nbytes_left < 0:
            log('recieved quit message')
            break

        msg_chunks = []
        while nbytes_left:
            msg_chunk = conn.recv(nbytes_left)
            if len(msg_chunk) == 0:
                log('unable to recieve full message')
                break
            else:
                log(len(msg_chunk), 'ok')
                nbytes_left -= len(msg_chunk)
                msg_chunks.append(msg_chunk)

        print(b''.join(msg_chunks).decode('utf8'))
