import socket
import threading
import sockmsg
import sys
from contextlib import closing

MESSAGE_LINE_PROMPT = '> '

def main():
    server_socket = connect_client(sys.argv[1], sys.argv[2], int(sys.argv[3]))
    with closing(server_socket):
        printer_thread = threading.Thread(target=incoming_messages_printer, args=(server_socket,))
        printer_thread.daemon = True
        printer_thread.start()
        user_messages_scanner(server_socket)

def connect_client(nickname, server_host, server_port):
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_socket.connect((server_host, server_port))
    sockmsg.OutgoingJson(['IDENT', nickname]).send(server_socket)
    handle_server_command(sockmsg.IncomingJson().recv(server_socket))
    return server_socket

def incoming_messages_printer(server_socket):
    while True:
        try:
            username, message = sockmsg.IncomingJson().recv(server_socket)
        except ValueError:
            print('malformed message format', file=sys.stderr)
        else:
            print('{0}: {1}'.format(username, message))

def user_messages_scanner(server_socket):
    while True:
        message_text = input(MESSAGE_LINE_PROMPT)
        if message_text.startswith(':'):
            handle_user_command(message_text)
        else:
            sockmsg.OutgoingJson(message_text).send(server_socket)

def handle_server_command(command):
    if type(command) != list:
        print('server issued unknown command', file=sys.stderr)
        return
    if command[0] == 'ACCEPTED':
        print('connected to', command[1])
    elif command == ['TAKEN']:
        print('choose another nickname and connect again', file=sys.stderr)
        sys.exit(-1)
    else:
        print('server issued unknown command', file=sys.stderr)

def handle_user_command(command):
    if command == ':q': # quit
        sys.exit(0)
    else:
        print(':q -- quit', file=sys.stderr)

if __name__ == '__main__':
    main()
